/* PROGRAM: aads (Ann Arbor Dating Service)

   AUTHOR:
   UNIQUENAME: 
   GSI NAME:
   EECS/CS 183
   Project Number:
   Due Date:
   
   SUMMARY

     This program reads in a data file containing one or more client records.
     Each ClientRec is a member of the dating service and has been given an ID.
     This program allows the display of ClientRec information as well as basic
     matching logic to match up compatible dating participants.

   INPUT
   
     The input comes from a file containing one or more client records.  Each
     record has the following format:

       <sex><blank><ID number><blank><name> 
       <age><blank><occupation> 
       <yearly income> 
       <major hobby> 
       <minor hobby> 

     The data types are as follows:   

       <sex> one character, 'M' or 'F' 
       <ID number> a 9-digit integer 
       <name> string 
       <age> an integer 
       <occupation> string 
       <yearly income> a real number 
       <major hobby> string 
       <minor hobby> string 

   OUTPUT
   
     The output goes to the console in a relatively "raw" format.  Nothing is
     specially formatted, except save the income.  Some I/O manipulation
     functions are used for income to make its huge value presentable.

   DATA STRUCTURES
   
     The record data for each ClientRec is stored in an instance of the
     'ClientRec' structure (user-defined data type).

   ASSUMPTIONS
   
     The data file is assumed to be correctly formatted.

*/

// include header files
#include <string>			// for all string variables
#include <iostream>			// for console input and output
#include <fstream>			// for loading data from file
#include <iomanip>			// to display decimal values correctly

// use standard namespace
using namespace std;

// global constants
const int		MAX_NUM_CLIENTS = 20,	// maximum number of clients to keep
										// records for
				CLIENT_NOT_FOUND = -1;	// invalid index value meaning a record
										// was not found
const string	STOP_CMD = "stop",		// 'stop' command
				SHOW_CMD = "show",		// 'show' command
				MATCH_CMD = "match";	// 'match' command

// ClientRec data structure type
typedef struct
{
	char			Sex;		// 'M' or 'F'
	unsigned long	ID;			// 9-Digit identification number
	string			Name;		// Name
	int				Age;		// Age
	string			Occup;		// Occupation
	double			Income;		// Yearly Income
	string			MajHobby;	// Major Hobby
	string			MinHobby;	// Minor Hobby
} ClientRec;

// array of client records type
typedef ClientRec ClientRecs[];

// function prototypes
void ProcessCommand(string Cmd, ClientRecs Clients, int NumClients);
void SortClientsAsc(ClientRecs Clients, int NumClients);
void CopyClient(ClientRec &Src, ClientRec &Dest);
int LoadClientsFile(string FileName, ClientRecs Clients);
void ShowClient(ClientRec &Client);
void MatchClient(ClientRec &Client, ClientRecs Clients, int NumClients);
int FindClient(unsigned long ID, ClientRecs Clients, int NumClients);






/* ************************************************************************** */
int main
  ()
/* This is the program entrance point.  It takes no additional input and
   always returns 0 (success).  The program itself may return EXIT_FAILURE,
   because 'exit()' is used in 'LoadClientsFile()'.
*/
{   // main

	// local constants
	const string Prompt					// prompt to display
		= "> ";
	const string DefaultFileName		// default data file name
		= "clients.txt";			

	// local variables
	ClientRec Clients[MAX_NUM_CLIENTS];	// array of 'ClientRec' structs
	int NumClients;						// size of 'Clients' array
	string Cmd;							// string to store user command

	// set flags to output floating-point numbers appropriately
	cout.setf(ios::fixed,ios::floatfield);
	cout.setf(ios::left,ios::adjustfield);

	// load ClientRec data from file
	NumClients = LoadClientsFile(DefaultFileName, Clients);

	// sort array of client records in ascending order (based on ID)
	SortClientsAsc(Clients, NumClients);

	// output header for sorted records
	cout << "***************************************"
	     << "***************************************" << endl
		 << "Sorted Records" << endl
		 << "***************************************"
	     << "***************************************" << endl
	     << endl;

	// show each ClientRec in 'Clients' array
	for (int Index = 0; Index < NumClients; Index++)
		ShowClient(Clients[Index]);

	// output footer for sorted records
	cout << "***************************************"
	     << "***************************************" << endl
		 << "End of Sorted Records" << endl
		 << "***************************************"
	     << "***************************************" << endl
	     << endl;

	// user command input loop
	do
	{   // loop until 'stop' command is entered

		// display prompt
		cout << Prompt;

		// get user command (stores the first word in 'Cmd')
		cin >> Cmd;

		// process user command on records array (Clients)		
		ProcessCommand(Cmd, Clients, NumClients);

		// clear input buffer (just in case there is left-over data in it)
		while (cin.rdbuf()->in_avail())
			cin.rdbuf()->stossc();

	}  // loop until 'stop' command is entered
	while (Cmd != STOP_CMD); 

    // return success
	return 0;
}   // main






/* ************************************************************************** */
void ProcessCommand
  (string Cmd,			// command to process
  ClientRecs Clients,	// array of 'ClientRec' structures to use with command
  int NumClients)		// number of subscripts in 'Clients' array
/* Processes a user command.  The 'Cmd' parameter stores the user command,
   'Clients' is an array of 'ClientRec' structures for the command to be used on
   (if it involves ClientRec data), and 'NumClients' is the number of used
   subscripts in the 'Clients' array.
*/
{   // ProcessCommand

	// local constants
	const string MSG_UNKNOWN_COMMAND	// error message on unknown command
		= "Unknown command!";

	// 'stop' command was entered															
	if (Cmd == STOP_CMD)
	{
		// do nothing
	}
	// 'show' or 'match' command was entered
	else if (Cmd == SHOW_CMD || Cmd == MATCH_CMD)
	{
		unsigned long ID;				// stores ID of desired ClientRec

		// get ID of desired ClientRec
		cin >> ID;

		// get index of desired ClientRec in 'Clients' array
		int Index = FindClient(ID, Clients, NumClients);

		// if index acquired, pass to specific command function
		if (Index != CLIENT_NOT_FOUND)
		{
			// if 'show' command was entered, call 'ShowClient()'
			if (Cmd == SHOW_CMD)
				ShowClient(Clients[Index]);
			// if 'match' command was entered, call 'MatchClient()'
			else
				MatchClient(Clients[Index], Clients, NumClients);
		}
	}
	// something ELSE was entered
	else
	{
		cout << MSG_UNKNOWN_COMMAND << endl;
	}
}   // ProcessCommand






/* ************************************************************************** */
void SortClientsAsc
  (ClientRecs Clients,	// array of 'ClientRec' structures to sort
  int NumClients)		// number of subscripts in 'Clients' array
/* Sorts the client records in the 'Clients' array (ClientRec structures) in
   ascending order based on their ID field.  This uses the simple bubble-
   sort algorithm.
*/
{   // SortClientsAsc

	// local variables
	ClientRec Temp;		// temporary 'ClientRec' variable for sorting purposes

	for (int TopIndex = 0; TopIndex < NumClients; TopIndex++)
	{   // for 0 to number of clients

		for (int SubIndex = 1; SubIndex < NumClients; SubIndex++)
		{   // for 1 to number of clients

			// switch ClientRecs if current ClientRec subscript has a lower ID
			// than the previous ClientRec subscript
			if (Clients[SubIndex].ID < Clients[SubIndex-1].ID)
			{
				// use 'Temp' to hold current ClientRec subscript while switching
				// the previous ClientRec subscript
				Temp = Clients[SubIndex];
				Clients[SubIndex] = Clients[SubIndex-1];
				Clients[SubIndex-1] = Temp;
			}

		}   // for 1 to number of clients

	}   // for 0 to number of clients

}   // SortClientsAsc






/* ************************************************************************** */
int LoadClientsFile
  (string FileName,		// file name of data file to load records from
  ClientRecs Clients)	// array of 'ClientRec' structures to store records in
/* Loads all the client records from the data file specified by 'FileName'.
   The records are placed in sequence into the 'Clients' array and the
   number of clients loaded is returned.
*/
{   // LoadClientsFile

	// local constants
	const string MSG_OPEN_FAIL		// error message when file can't be opened
		= "Couldn't open client records data file!";
	const char	SEX_MALE = 'M',		// data representation of male sex
				SEX_FEMALE = 'F';	// data representation of female sex

	// local variables
	int NumClients = 0;				// number of clients loaded from file
	fstream File;					// file stream object for file i/o
	string TempStr;					// temporary string to read in whole lines

	// open data file specified by 'FileName'.  Must use 'c_str()' because there
	// is no implicit conversion to 'const char*' from C++ string
	File.open(FileName.c_str(), ios::in);

	// check for open failure, if so print message and exit
	if (File.fail())
	{
		cout << MSG_OPEN_FAIL << endl;
		exit(EXIT_FAILURE);
	}

	// output header for reading file records
	cout << "***************************************"
	     << "***************************************" << endl
		 << "Reading Records from File" << endl
		 << "***************************************"
	     << "***************************************" << endl
	     << endl;

	// continue reading file until EOF marker is reached OR until the 'Clients'
	// array has been completely filled
	while (!File.eof() && NumClients < MAX_NUM_CLIENTS)
	{
		// get sex code (char)
		File.get(Clients[NumClients].Sex);

		// if a blank space or additional character is at end of file, it can
		// mess up the input.  Break out if sex is not valid.
		if (Clients[NumClients].Sex != SEX_MALE &&
			Clients[NumClients].Sex != SEX_FEMALE)
			break;

		// get ID (unsigned long)
		File >> Clients[NumClients].ID;

		// get name (string)
		File.ignore(1);
		getline(File, Clients[NumClients].Name);

		// get age (int)
		File >> Clients[NumClients].Age;

		// get occupation (string)
		File.ignore(1);
		getline(File, Clients[NumClients].Occup);

		// get income (double)
		getline(File, TempStr);
		Clients[NumClients].Income = strtod(TempStr.c_str(), NULL);

		// get hobbies (string,string)
		getline(File, Clients[NumClients].MajHobby);
		getline(File, Clients[NumClients].MinHobby);

		// show client record loaded
		ShowClient(Clients[NumClients]);

		// move to next subscript in 'Clients' array
		NumClients++;
	}

	// output footer for reading records
	cout << "***************************************"
	     << "***************************************" << endl
		 << "Finished Reading Records from File" << endl
		 << "***************************************"
	     << "***************************************" << endl
	     << endl;

	// return number of client records loaded
	return NumClients;
}   // LoadClientsFile






/* ************************************************************************** */
void ShowClient
  (ClientRec &Client)		// Client object to show
/* Shows all the data in the ClientRec object (Client).
*/
{   // ShowClient
	cout	<< "ID = "				<< Client.ID		<< endl
			<< "Name = "			<< Client.Name		<< endl
			<< "Sex = "				<< Client.Sex		<< endl
			<< "Age = "				<< Client.Age		<< endl
			<< "Occupation = "		<< Client.Occup		<< endl
			<< "Yearly Income = "
				<< setw(10) << setprecision(2)
				<< Client.Income	<< endl
			<< "Major Hobby = "		<< Client.MajHobby	<< endl
			<< "Minor Hobby = "		<< Client.MinHobby	<< endl
			<< endl;
}    // ShowClient






/* ************************************************************************** */
void MatchClient
  (ClientRec &Client,	// ClientRec object to match up
   ClientRecs Clients,	// array of 'ClientRec' structures to match in
   int NumClients)		// number of subscripts in 'Clients' array
/* Attempts to match ClientRecs from the 'Clients' array with the ClientRec
   specified by 'Client' using basic compatibility testing.
*/
{   // MatchClient

	// local constants
	const int MAX_AGE_DIFF				// max age difference between compatibles
		= 10;
	const double MAX_INCOME_DIFF		// max income difference between
		= 10000.0;						// compatibles
	const string MSG_NO_MATCHES			// error message if no matches are made
		= "No matches could be made for this client!";

	// local variables
	int Matches							// count of matches made
		= 0;

	for (int Index = 0; Index < NumClients; Index++)
	{   // for 0 to number of clients
		if (
			// compatible if within specified age and income range ...
			((Clients[Index].Age < Client.Age + MAX_AGE_DIFF &&
			Clients[Index].Age > Client.Age - MAX_AGE_DIFF &&
			Clients[Index].Income < Client.Income + MAX_INCOME_DIFF &&
			Clients[Index].Income > Client.Income - MAX_INCOME_DIFF) ||

			// OR if hobbies are identical
			(Clients[Index].MajHobby == Client.MajHobby &&
			Clients[Index].MinHobby == Client.MinHobby)) &&

			// but ONLY if of opposite sex
			Clients[Index].Sex != Client.Sex)
		{
			// display compatible ClientRec
			ShowClient(Clients[Index]);

			// increment number of matches made
			Matches++;
		}
	}   // for 0 to number of clients

    // display error if no matches were made
	if (!Matches)
		cout << MSG_NO_MATCHES << endl;
}   // MatchClient






/* ************************************************************************** */
int FindClient
  (unsigned long ID,	// ID to match
  ClientRecs Clients,	// array of 'ClientRec' structures to search in
  int NumClients)		// number of subscripts in 'Clients' array
/* Finds ClientRec structure in 'Clients' array by matching up their 'ID' with
   the one passed in.  Returns the index of the ClientRec object in the
   'Clients' array.
*/
{   // FindClient

	// local constants
	const string MSG_NOT_FOUND	// error to display if ClientRec not found
		= "The following Client ID was not found: ";

	// local variables
	int FoundIndex				// found ClientRec index (defaults to not found)
		= CLIENT_NOT_FOUND;

	for (int Index = 0; FoundIndex == CLIENT_NOT_FOUND && Index < NumClients; Index++)
	{   // for 0 to number of clients while ClientRec not found

		// if ID's match ...
		if (Clients[Index].ID == ID)
		{
			// set 'FoundIndex' ... this will cause the loop to end
			FoundIndex = Index;
		}

	}   // for 0 to number of clients while ClientRec not found

	// if no matching ID was found, report error
	if (FoundIndex == CLIENT_NOT_FOUND)
		cout << MSG_NOT_FOUND << ID << endl;

	// return index found (or CLIENT_NOT_FOUND)
	return FoundIndex;
}   // FindClient