/*
Programming Assignment 5: Bank Wait Times
Point Value: 100 points
Project Due Date: Wednesday April 3, 2002

Learning Objectives

	-> To write a program which requires the manipulation of parallel
		one-dimensional arrays
	-> To calculate some commonly used statistics and draw a histogram

-[Problem Statement]-

The Very Small Bank wants information on how long its customers are waiting for
service at drive-in windows. Employees have collected information, which your
program will read from a text file. The data consists of the time the customer
arrived in hours, minutes and seconds, the time the customer was actually
served, and the ID number of the teller. The first digit of the teller ID
indicates which branch of the bank was visited, and the next 2 digits identify
the teller. The bank is open from 9 AM to 5 PM. You can assume that a customer
is always served on the same day he arrives (if not, the bank would go out of
business in about a week!).

-[Input]-

Data file: p5w02data.txt, containing an unknown number of lines, each containing
these items starting in the columns indicated:

Column 1: teller ID (a 3-digit integer)
Column 10: arrival time (as a 6-digit integer; see below)
Column 25: service time (same format as arrival time)

All times are given based on a 24-hour clock, with no colons. For example,
1:02:01 PM is represented in the data file as 130201. You can assume a maximum
of 50 lines in the file, and that all data is present, valid, and correctly
formatted

Processing

Your program must read in all input data, then use it to calculate each wait
time in seconds (an integer), the mean wait time (a real), the standard
deviation of the wait times (real), the shortest and longest wait times
(integers), and the range of the wait times (integer). Note all wait time data
is to be processed in seconds.

*/

// =============================================================================
// Bank Wait Times
// By <name>
//
// Reads in a data file containing the service wait times for customers, then
// generates a useful report based on that raw data.
// =============================================================================

//
// include header files
//
#include <string>   // for all string variables (string)
#include <iostream> // for console output (cout)
#include <fstream>  // for loading data from file
#include <cstdlib>  // C stdlib.h functions
#include <cmath>	// C math.h functions (sqrt, fabs)

// use standard namespace
using namespace std;

//
// Level of frustration for a particular customer.
//
enum Frustration
{
	Amazed		= 0,	// wait <= (mean - SD)
	Pleased		= 1,	// (mean - SD) < wait < mean
	Calm		= 2,	// mean <= wait < (mean + SD)
	Irritated	= 3,	// (mean + SD) <= wait < (mean + 2*SD)
	Berserk		= 4,	// (mean + 2*SD) <= wait

	MaxFrustrationLevels	= 5		// constant to track number of levels
};

//
// Function Prototypes
//
void ReportStatistics(int Seconds[], int Count);
void PrintHistogram(int Seconds[], int Count, double Mean, double SD);
bool ReadLine(ifstream &InFile, int &TellerID, int &ArrivalTime, int &ServiceTime);
Frustration GetFrustration(int WaitTime, double Mean, double SD);
int CalculateSeconds(int ArrivalTime);
int GetLongestWait(int Times[], int Count);
int GetShortestWait(int Times[], int Count);
double GetMean(int Seconds[], int Count);
double GetStandardDeviation(int Seconds[], int Count, double Mean);

// =============================================================================
// Program entry-point function.
// =============================================================================
int main()
{
	//
	// Customer data
	//
	int	aiTeller[50];		// id of teller who served this customer
	int aiArrival[50];		// arrival time (6 digit integer: HHMMSS)
	int aiService[50];		// service time (6 digit integer: HHMMSS)
	int aiWait[50];			// calculated wait time in number of seconds
	int iNumCustomers = 0;	// number of customers

	//
	// File data
	//
	ifstream InFile;		// file stream object to load data file
	string sFileName = "p5w02data.txt";

	// open data file
	InFile.open(sFileName.c_str());

	// process on success
	if (!InFile.fail())
	{
		//
		// output echo-print input header
		//
		cout << "Reading Data File \"" << sFileName << "\":" << endl << endl
			<< "Customer/Line\tTeller\tArrival\tService\tWait Time (computed)"
			<< endl;

		//
		// Read in all customer data and echo-print it.
		//
		while (ReadLine(InFile,
				aiTeller[iNumCustomers],
				aiArrival[iNumCustomers],
				aiService[iNumCustomers]))
		{
			// calculate the wait time for this customer
			aiWait[iNumCustomers] = CalculateSeconds(aiService[iNumCustomers]) -
									CalculateSeconds(aiArrival[iNumCustomers]);

			// echo-print input data
			cout << "Customer " << (iNumCustomers+1) << ":\t"
				<< aiTeller[iNumCustomers] << "\t"
				<< aiArrival[iNumCustomers] << "\t"
				<< aiService[iNumCustomers] << "\t"
				<< aiWait[iNumCustomers] << endl;

			// increase customer index for parallel arrays
			iNumCustomers++;
		}

		//
		// Generate and output statics report.
		//
		ReportStatistics(aiWait, iNumCustomers);

	} // if (successfully opened data file)

	// else report error and exit
	else
	{
		cout << "Failed to open \"" << sFileName << "\"!" << endl;
	}

	// close data file
	InFile.close();

	return 0;
}

// =============================================================================
// Reports statistical information about the array of wait times (Seconds[])
// read in, including longest wait time, shortest wait time, mean time, standard
// deviation, range, and a histogram of frustration levels.
// =============================================================================
void ReportStatistics(int Seconds[], int Count)
{
	double nMean;			// mean of wait time
	double nSD;				// standard deviation of wait times
	int iRange;				// range of wait times
	int iLongestWait;		// longest wait
	int iShortestWait;		// shortest wait

	//
	// get longest wait time
	//
	iLongestWait = GetLongestWait(Seconds, Count);

	//
	// get shortest wait time
	//
	iShortestWait = GetShortestWait(Seconds, Count);

	//
	// calculate mean
	//
	nMean = GetMean(Seconds, Count);

	//
	// calculate standard deviation
	//
	nSD = GetStandardDeviation(Seconds, Count, nMean);

	//
	// calculate range = difference between maximum value and minimum value
	//
	iRange = iLongestWait - iShortestWait;

	//
	// print statistics results
	//
	cout << endl
		<< "Statistics:" << endl
		<< endl
		<< "Number of customers in data file (lines): " << Count
		<< endl
		<< "Mean wait time: " << nMean << " seconds" << endl
		<< "Standard deviation of wait times: " << nSD << endl
		<< "Shortest wait time: " << iShortestWait << " seconds" << endl
		<< "Longest wait time: " << iLongestWait << " seconds" << endl
		<< "Range of wait times: " << iRange << endl;

	//
	// output histogram
	//
	PrintHistogram(Seconds, Count, nMean, nSD);
}

// =============================================================================
// Prints a histogram of frustration levels.
// =============================================================================
void PrintHistogram(int Seconds[], int Count, double Mean, double SD)
{
	int Tally[MaxFrustrationLevels+1];	// stores count of each frust. level
	int i;								// generic index variable
	int a;								// asterisk counter

	// reset tally array
	for (i = 0; i < MaxFrustrationLevels; i++)
		Tally[i] = 0;

	// tally each frustration level by adding to the count in each subscript
	for (i = 0; i < Count; i++)
		Tally[GetFrustration(Seconds[i], Mean, SD)]++;

	//
	// output histogram header
	//
	cout << endl;
	cout << "Histogram of Frustration Levels:"
		<< endl << endl;
	cout << "          0    5   10   15   20   25   30   35   40   45   50"
		<< endl;
	cout << "          |----+----|----+----|----+----|----+----|----+----|"
		<< endl;

	//
	// output the amount of each frustration level in histogram
	//
	for (i = 0; i < MaxFrustrationLevels; i++)
	{
		//
		// output ten character name of frustration level, right padded with
		// spaces.
		//
		switch (i)
		{
			case Amazed:
				cout << "Amazed    ";
				break;

			case Pleased:
				cout << "Pleased   ";
				break;

			case Calm:
				cout << "Calm      ";
				break;

			case Irritated:
				cout << "Irritated ";
				break;

			case Berserk:
				cout << "Berserk   ";
				break;
			default:
				cout << "Unknown   ";
		}

		//
		// output amount of asterisks indicating level (zero counts as one
		// asterisk on this histogram).
		//
		for (a = 0; a <= Tally[i] && a <= 50; a++)
			cout << '*';
		cout << endl;
	}

	//
	// output histogram footer
	//
	cout << "          |----+----|----+----|----+----|----+----|----+----|"
		<< endl;
	cout << "          0    5   10   15   20   25   30   35   40   45   50"
		<< endl;
}

// =============================================================================
// Reads a single line from the data file and places the corresponding values
// into the reference parameters passed in.  If the line could not be read OR
// there were no more lines to be read, the function returns false.
// =============================================================================
bool ReadLine(ifstream &InFile, int &TellerID, int &ArrivalTime, int &ServiceTime)
{
	InFile >> TellerID >> ArrivalTime >> ServiceTime;

	if (InFile.fail() || InFile.eof())
		return false;

	return true;
}

// =============================================================================
// Determines the frustration level of the customer based on the wait time
// passed in (see 'Frustration' enum):
//
//	Wait Time 							Frustration Level
//	--------- 							-----------------
//	wait <= (mean - SD) 					Amazed
//	(mean - SD) < wait < mean 				Pleased
//	mean <= wait < (mean + SD) 				Calm
//	(mean + SD) <= wait < (mean + 2*SD) 	Irritated
//	(mean + 2*SD) <= wait 					Berserk
//
// If a specific frustration level cannot be determined then
// 'MaxFrustrationLevels' is returned.
// =============================================================================
Frustration GetFrustration(int WaitTime, double Mean, double SD)
{
	Frustration f = MaxFrustrationLevels;

	if (WaitTime <= (int)(Mean - SD))
	{
		f = Amazed;
	}
	else if ((int)(Mean - SD) < WaitTime && WaitTime < Mean)
	{
		f = Pleased;
	}
	else if ((int)(Mean) < WaitTime && WaitTime < (int)(Mean + SD))
	{
		f = Calm;
	}
	else if ((int)(Mean + SD) <= WaitTime && WaitTime < (int)(Mean + 2*SD))
	{
		f = Irritated;
	}
	else if ((int)(Mean + 2*SD) <= WaitTime)
	{
		f = Berserk;
	}

	return f;
}

// =============================================================================
// Calculates and returns the number of seconds represented by the six-digit
// integer time passed in, which is in the format HHMMSS (24-hour clock time).
// =============================================================================
int CalculateSeconds(int Time)
{
	//
	// The calculations to break up the digits into their component (hours,
	// minutes, seconds) parts have been seperated into individual variables
	// and algorithms for clarity.
	//
	int Hours = Time / 10000;
	int Minutes = (Time - (Hours * 10000)) / 100;
	int Seconds = Time - (Hours * 10000) - (Minutes * 100);

	//
	// Uncomment the following for debugging (to make sure the calculations are
	// correct):
	//
	//cout << "Calculated: " << Hours << ':' << Minutes << ':' << Seconds
	//	<< " from " << Time << endl;

	// Return the accumulative number of seconds.
	return (Hours * 3600) + (Minutes * 60) + Seconds;
}

// =============================================================================
// Discovers the longest wait time in the array of times (in seconds) and
// returns it.
// =============================================================================
int GetLongestWait(int Seconds[], int Count)
{
	int i, Longest = Seconds[0];

	for (i = 1; i < Count; i++)
	{
		if (Seconds[i] > Longest)
			Longest = Seconds[i];
	}

	return Longest;
}

// =============================================================================
// Discovers the shortest wait time in the array of times (in seconds) and
// returns it.
// =============================================================================
int GetShortestWait(int Seconds[], int Count)
{
	int i, Shortest = Seconds[0];

	for (i = 1; i < Count; i++)
	{
		if (Seconds[i] < Shortest)
			Shortest = Seconds[i];
	}

	return Shortest;
}

// =============================================================================
// Calculates the mean (average) of the wait times.
//
// mean = (sum of all values) divided by (number of values)
// =============================================================================
double GetMean(int Seconds[], int Count)
{
	double nMean = 0.0;
	int i;

	// add up all the values
	for (i = 0; i < Count; i++)
		nMean += (double)Seconds[i];

	// return the mean after dividing by number of values
	return (nMean /= Count);
}

// =============================================================================
// Calculates the standard deviation.
//
// standard deviation = the square root of:
// 	(sum of the squares of the differences between each value and
// 	the mean) divided by (number of values - 1)
// =============================================================================
double GetStandardDeviation(int Seconds[], int Count, double Mean)
{
	double nSumSquare = 0.0;
	double nMeanDiff = 0.0;
	int i;

	for (i = 0; i < Count; i++)
	{
		// get the difference between the wait time and the mean wait time.
		nMeanDiff = (double)Seconds[i] - Mean;

		// add the square of the previous difference to the sum of squares.
		nSumSquare += (nMeanDiff * nMeanDiff);
	}

	// divide the sum of squares by the number of values minus one
	nSumSquare /= (double)(Count - 1);

	// return the square root of the sum of squares which will now represent
	// the "mean of means" or standard deviation.
	return sqrt(nSumSquare);
}