/*****************************************************************************

	[help.cpp] Help function implementation(s)
	By Jeff Leung

*****************************************************************************/
#include "help.h"
#include "misc.h"

//_[printHelp()]______________________________________________________________
//
// DESCRIPTION:
//  Prints a list of commands or help for a specific command.
//
// PARAMETERS:
//  aWords ................ array of words
//  i ..................... starting index (subscript should be 'help')
//
// RETURN VALUE(void):
//  None.
//
void printHelp(vector<string> &aWords, unsigned int i)
{
	i = nextWord(aWords, i+1);

	//_[output general list]__________________________________________________
	if (i == 0)
	{
		cout
			<< "Commands:" << endl
			<< endl
			<< "    tally .... statistics" << endl
			<< "    load ..... retrieve data from file" << endl
			<< "    save ..... store data to file" << endl
			<< "    list ..... display list of books or clients" << endl
			<< "    show ..... shows detailed info about a book or client" << endl
			<< "    new ...... create a new book or client" << endl
			<< "    delete ... delete a book or client" << endl
			<< "    borrow ... borrow a book by a client" << endl
			<< "    return ... return a book borrowed by a client" << endl
			<< "    help ..... show this listing or help with a specific command" << endl
			<< "    quit ..... end program" << endl
			<< endl
			<< "For information on a particular command use 'help <command>'"
			<< endl;
	}
	//_[output 'help' command information]____________________________________
	else if (aWords[i] == "help")
	{
		cout
			<< "'help' Command:" << endl
			<< endl
			<< "Provides a listing of all commands or detailed help on a particular one." << endl
			<< endl
			<< "USAGE:" << endl
			<< "\thelp <command>" << endl
			<< endl;
	}
	//_[output 'quit' command information]____________________________________
	else if (aWords[i] == "quit")
	{
		cout
			<< "'quit' Command:" << endl
			<< endl
			<< "Ends the program.  Make sure that you use 'save' if you want to keep any data you've modified." << endl
			<< endl
			<< "USAGE:" << endl
			<< "\tquit" << endl
			<< endl;
	}
	//_[output 'tally' command information]___________________________________
	else if (aWords[i] == "tally")
	{
		cout
			<< "'tally' Command:" << endl
			<< endl
			<< "Tallies any number of things based on the filter given (type 'help filters' for more info)." << endl
			<< endl
			<< "USAGE:" << endl
			<< "\ttally <filter>" << endl
			<< endl;
	}
	//_[output 'list' command information]____________________________________
	else if (aWords[i] == "list")
	{
		cout
			<< "'list' Command:" << endl
			<< endl
			<< "Lists out a number of books or clients (but not both) based on the filter given (type 'help filters' for more info) and the start index (<start>).  The <start> parameter is optional." << endl
			<< endl
			<< "USAGE:" << endl
			<< "\tlist <filter> <start>" << endl
			<< endl;
	}
	//_[output 'show' command information]____________________________________
	else if (aWords[i] == "show")
	{
		cout
			<< "'show' Command:" << endl
			<< endl
			<< "Displays information about a book or client." << endl
			<< endl
			<< "USAGE:" << endl
			<< "\tshow (\"book\"/\"client\") (<isbn>/<lastname>,<firstname>)" << endl
			<< endl;
	}
	//_[output 'load' command information]____________________________________
	else if (aWords[i] == "load")
	{
		cout
			<< "'load' Command:" << endl
			<< endl
			<< "Loads data from an XML file." << endl
			<< endl
			<< "USAGE:" << endl
			<< "\tload <filename>.xml" << endl
			<< endl;
	}
	//_[output 'save' command information]____________________________________
	else if (aWords[i] == "save")
	{
		cout
			<< "'save' Command:" << endl
			<< endl
			<< "Saves all data to an XML file." << endl
			<< endl
			<< "USAGE:" << endl
			<< "\tsave <filename>.xml" << endl
			<< endl;
	}
	//_[output 'new' command information]_____________________________________
	else if (aWords[i] == "new")
	{
		cout
			<< "'new' Command:" << endl
			<< endl
			<< "Creates a new book or client." << endl
			<< endl
			<< "USAGE:" << endl
			<< "\tnew (\"book\"/\"client\")" << endl
			<< endl;
	}
	//_[output 'delete' command information]__________________________________
	else if (aWords[i] == "delete")
	{
		cout
			<< "'delete' Command:" << endl
			<< endl
			<< "Deletes a specified book or client." << endl
			<< endl
			<< "USAGE:" << endl
			<< "\tdelete (\"book\"/\"client\") (<book>/<client>)" << endl
			<< endl;
	}
	//_[output 'borrow' command information]__________________________________
	else if (aWords[i] == "borrow")
	{
		cout
			<< "'borrow' Command:" << endl
			<< endl
			<< "" << endl
			<< endl
			<< "USAGE:" << endl
			<< "\tborrow <book> <client>" << endl
			<< endl;
	}
	//_[output 'return' command information]__________________________________
	else if (aWords[i] == "return")
	{
		cout
			<< "'return' Command:" << endl
			<< endl
			<< "" << endl
			<< endl
			<< "USAGE:" << endl
			<< "\treturn <book> <client>" << endl
			<< endl;
	}
	//_[output filters information]___________________________________________
	else if (aWords[i] == "filters")
	{
		cout
			<< "Filters Available:" << endl
			<< endl
			<< "    all ............. all books & clients" << endl
			<< "    books ........... all books" << endl
			<< "    inbooks ......... books that are not being borrowed" << endl
			<< "    outbooks ........ books that are borrowed" << endl
			<< "    notduebooks ..... books that are borrowed, but not due" << endl
			<< "    duebooks ........ books that are borrowed and due" << endl
			<< "    clients ......... all clients" << endl
			<< "    emptyclients .... clients that aren't borrowing any books" << endl
			<< "    bookclients ..... clients that are borrowing books" << endl
			<< "    goodclients ..... clients that are borrowing books, but none are due" << endl
			<< "    badclients ...... clients that are borrowing with books due" << endl
			<< endl
			<< "To use multiple filters, put a bar (|) between them.  For example " << endl
			<< "for all bad clients and due books: " << endl
			<< endl
			<< "  badclients|duebooks" << endl
			<< endl;
	}
	//_[unknown command]______________________________________________________
	else
	{
		cout
			<< "Unknown command '" << aWords[i]
			<< "'.  Try just typing 'help'" << endl;
	}
}