/*****************************************************************************

	[misc.cpp] Miscelaneous function implementations
	By Jeff Leung

*****************************************************************************/
#include "misc.h"
#include "bookshop.h"

//_[nextWord()]_______________________________________________________________
//
// DESCRIPTION:
//  Returns the index of the next non-blank word in the vector, starting with
//  the index iCur.
//
// PARAMETERS:
//  aWordVec .............. vector of strings
//  iCur .................. current index into aWordVec
//
// RETURN VALUE (unsigned int):
//  See description.  Returns 0 if past end of vector.
//
unsigned int nextWord(vector<string> &aWordVec, unsigned int iCur)
{
	while (iCur < aWordVec.size() && aWordVec[iCur] == "")
	{
		iCur++;
	} 

	if (iCur >= aWordVec.size())
		return 0;

	return iCur;
}

//_[getTodaysDate()]__________________________________________________________
//
// DESCRIPTION:
//  Gets today's date in local time in the format DD MMM YYYY (01 JAN 2001).
//
// PARAMETERS:
//  None.
//
// RETURN VALUE (string):
//  See description.
//
string getTodaysDate(void)
{
	char sStore[16], *pt;
	time_t timer;
	string sDate;

	timer = time(NULL);

	strftime(sStore, 15, "%d %b %Y", localtime(&timer));

	for (pt = sStore; *pt; pt++)
		*pt = toupper(*pt);

	sDate = sStore;
	return sDate;
}

//_[isDateValid()]____________________________________________________________
//
// DESCRIPTION:
//  Determines if the specified date (sDate) is valid.
//
// PARAMETERS:
//  sDate ................. date to check
//
// RETURN VALUE (bool):
//  Returns 'true' if the date string is valid, otherwise returns 'false'.
//
bool isDateValid(string &sDate)
{
	char sMonthC[4], *pt;
	string sMonth;
	unsigned int iDay, iYear;

	//_[check length of date string]__________________________________________
	if (sDate.length() != 11)
		return false;

	//_[check day]____________________________________________________________
	iDay = strtol(sDate.substr(0,2).c_str(), NULL, 10);

	if (iDay < 1 || iDay > 31)
		return false;

	//_[check year]___________________________________________________________
	iYear = strtol(sDate.substr(7,4).c_str(), NULL, 10);

	if (iYear < 2001)
		return false;

	//_[make sure month string is all upper-case]_____________________________
	strcpy(sMonthC, sDate.substr(3,3).c_str());
	for (pt = sMonthC; *pt; pt++)
		*pt = toupper(*pt);
	sMonth = sMonthC;

	//_[check month]__________________________________________________________
	if (sMonth != "JAN" &&
		sMonth != "FEB" &&
		sMonth != "MAR" &&
		sMonth != "APR" &&
		sMonth != "MAY" &&
		sMonth != "JUN" &&
		sMonth != "JUL" &&
		sMonth != "AUG" &&
		sMonth != "SEP" &&
		sMonth != "OCT" &&
		sMonth != "NOV" &&
		sMonth != "DEC")
		return false;

	return true;
}

//_[hasDatePast()]____________________________________________________________
//
// DESCRIPTION:
//  Determines if the specified date (sDate) is already past.
//
// PARAMETERS:
//  sDate ................. date to check
//
// RETURN VALUE (bool):
//  Returns 'true' if the date has past, otherwise returns 'false'.
//
bool hasDatePast(string &sDate)
{
	char sMonthC[4], *pt;
	time_t timer;
	struct tm *pToday;
	struct tm oDate;
	string sMonth;
	bool fPast = false;

	//_[make sure date is valid before proceeding]____________________________
	if (!isDateValid(sDate))
		return true;

	//_[get today's time]_____________________________________________________
	timer = time(NULL);
	pToday = localtime(&timer);

	//_[determine numeric day and year values]________________________________
	oDate.tm_mday = strtol(sDate.substr(0,2).c_str(), NULL, 10);
	oDate.tm_year = strtol(sDate.substr(7,4).c_str(), NULL, 10) - 1900;

	//_[make sure month string is all upper-case]_____________________________
	strcpy(sMonthC, sDate.substr(3,3).c_str());
	for (pt = sMonthC; *pt; pt++)
		*pt = toupper(*pt);
	sMonth = sMonthC;

	//_[determine numeric month value]________________________________________
	if (sMonth == "JAN")
		oDate.tm_mon = 0;
	else if (sMonth == "FEB")
		oDate.tm_mon = 1;
	else if (sMonth == "MAR")
		oDate.tm_mon = 2;
	else if (sMonth == "APR")
		oDate.tm_mon = 3;
	else if (sMonth == "MAY")
		oDate.tm_mon = 4;
	else if (sMonth == "JUN")
		oDate.tm_mon = 5;
	else if (sMonth == "JUL")
		oDate.tm_mon = 6;
	else if (sMonth == "AUG")
		oDate.tm_mon = 7;
	else if (sMonth == "SEP")
		oDate.tm_mon = 8;
	else if (sMonth == "OCT")
		oDate.tm_mon = 9;
	else if (sMonth == "NOV")
		oDate.tm_mon = 10;
	else
		oDate.tm_mon = 11;

	//_[compare dates]________________________________________________________
	if (pToday->tm_year > oDate.tm_year)
	{
		fPast = true;
	}
	else if (pToday->tm_year < oDate.tm_year)
	{
		fPast = false;
	}
	else
	{
		if (pToday->tm_mon > oDate.tm_mon)
		{
			fPast = true;
		}
		else if (pToday->tm_mon < oDate.tm_mon)
		{
			fPast = false;
		}
		else
		{
			if (pToday->tm_mday > oDate.tm_mday)
			{
				fPast = true;
			}
			else
			{
				fPast = false;
			}
		}
	}

	return fPast;
}

//_[stringFlatten()]__________________________________________________________
//
// DESCRIPTION:
//  Takes all special characters in a string and flattens them into something
//  that can easily be stored.  The special character becomes "&#n;" where
//  'n' is the number of the character unraveled into text.
//
// PARAMETERS:
//  sSource ............... source string to be flattened
//
// RETURN VALUE (string):
//  Returns the flattened string.
//
string stringFlatten(string &sSource)
{
	char sChar[8];
	string sRet;


	for (unsigned int i = 0; i < sSource.length(); i++)
	{
		if (!isalpha(sSource[i]) &&
			!isdigit(sSource[i]) &&
			sSource[i] != ' ')
		{
			sprintf(sChar, "%u", (unsigned int)sSource[i]);
			sRet += "&#";
			sRet += sChar;
			sRet += ";";
		}
		else
		{
			sRet += sSource[i];
		}
	}

	return sRet;
}


//_[getFilter()]______________________________________________________________
//
// DESCRIPTION:
//  Takes a string that represents all the different filters to be combined
//  together.
//
// PARAMETERS:
//  sSource ............... source string to be parsed for filters
//
// RETURN VALUE (BookShop::filter_type):
//  Returns the filter value.
//
int getFilter(string &sSource)
{
	int iFilter = (int)BookShop::None;
	unsigned int bi = 0, ei = 0;
	string sFilt;

	//_[iterate through filter string]________________________________________
	while (ei < sSource.length())
	{
		//-[get next filter string piece]-------------------------------------
		ei = sSource.find("|", bi);
		if (ei == string::npos)
			ei = sSource.length();
		sFilt = sSource.substr(bi, ei-bi);

		//-[combine appropriate numeric filter]-------------------------------
		if (sFilt == "all")
			iFilter |= (int)BookShop::All;
		else if (sFilt == "books")
			iFilter |= (int)BookShop::AllBooks;
		else if (sFilt == "inbooks")
			iFilter |= (int)BookShop::InBooks;
		else if (sFilt == "outbooks")
			iFilter |= (int)BookShop::OutBooks;
		else if (sFilt == "notduebooks")
			iFilter |= (int)BookShop::NotDueBooks;
		else if (sFilt == "duebooks")
			iFilter |= (int)BookShop::DueBooks;
		else if (sFilt == "clients")
			iFilter |= (int)BookShop::AllClients;
		else if (sFilt == "emptyclients")
			iFilter |= (int)BookShop::EmptyClients;
		else if (sFilt == "bookclients")
			iFilter |= (int)BookShop::BookClients;
		else if (sFilt == "goodclients")
			iFilter |= (int)BookShop::GoodClients;
		else if (sFilt == "badclients")
			iFilter |= (int)BookShop::BadClients;

		bi = ei + 1;
	}

	return iFilter;
}