//*[hangman.cpp]*******************************************************(author)*
//
//  0    Hangman
// -|-
//  |    This program randomly picks a word from a text file specified by the
// / \   user and allows the user to try to guess that word.  If the word is not
//       guessed in seven tries (head, neck, two arms, body, and two legs) the
//       game is over and the user has lost.
//
//******************************************************************************

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
// Header Files
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
#include <conio.h>			// gotoxy(), getch(), clrscr()
#include <iostream.h>		// cout
#include <fstream.h>		// ifstream
#include <stdlib.h>			// malloc(), rand()
#include <string.h>			// strcmp(), strncpy()
#include <time.h>			// time()
#include <ctype.h>			// isalpha()

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
// Function Prototypes
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
char *PickWord(ifstream &InFile);
void ClearScreen(void);
void DrawHangMan(int BadGuesses);
void DrawGameScreen(char *Guess, int BadGuesses);

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
// Function Definitions
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

//_[main()]_____________________________________________________________________
//
// Program entry-point.
//
int main(int argc, char *argv[])
{
	ifstream InFile;		// input file from which word is taken
	char *Word = NULL;		// word to be guessed
	char *Guess = NULL;		// currently guessed word (starts as all blanks)
	char Letter;			// guessed letter
	char PlayAgain;			// play again flag set by user after a game is over
	int BadGuesses;			// count of bad guesses made by user
	int Losses = 0;			// count of people hung
	int i;					// generic index variable
	int Found;				// indicates if guessed letter was found in word
	int Won;				// indicates winning status (yes/no)

	srand(time(NULL));

	//
	// check for appropriate number of arguments
	//
	if (argc < 2)
	{
		cout << "You must specify a filename: " << endl
			<< "    hangman.exe [filename.txt]" << endl
			<< endl;
		return 1;
	}

	//
	// attempt to open user-specified file
	//
	InFile.open(argv[1]);

	if (InFile.fail())
	{
		cout << "Failed to open \"" << argv[1] << "\"!" << endl;
		return 1;
	}

	//
	// main game loop
	//
	do
	{
		//
		// reset game data and free used memory
		//
		BadGuesses = 0;
		Won = 0;

		if (Guess != NULL)
		{
			free(Guess);
			Guess = NULL;
		}

		if (Word != NULL)
		{
			free(Word);
			Word = NULL;
		}

		//
		// pick word from file
		//
		Word = PickWord(InFile);
		if (Word == NULL)
		{
			InFile.close();
			cout << "Failed to get word from file (\"" << argv[1] << "\")!" << endl;
			return 1;
		}

		//
		// reset guess to a blank word of same length as 'Word'
		//
		Guess = (char*)malloc(strlen(Word)+1);

		if (Guess == NULL)
		{
			cout << "Failed to allocate space for guesses!" << endl;
			return 1;
		}

		memset(Guess, ' ', strlen(Word));
		Guess[strlen(Word)] = '\0';

		//
		// draw the screen to start
		//
		DrawGameScreen(Guess, BadGuesses);

		//
		// single game loop (goes until word is guessed OR all bad choices have
		// been used up).
		//
		do
		{
			// get letter choice by user
			Letter = tolower( (char)getch() );

			// check choice against word to guess
			Found = 0;
			for (i = 0; Word[i] && !Won; i++)
			{
				// found letter in word
				if (Word[i] == Letter)
				{
					Guess[i] = Word[i];
					Found = 1;

					// if we guessed the word we won!
					if (!strcmp(Guess, Word))
					{
						Won = 1;
					} // if guessed word

				} // if found letter in word

			} // loop through letters to guess

			// if didn't find letter, increment count of bad choices
			if (!Found)
			{
				BadGuesses++;
			}

			// draw the new screen
			DrawGameScreen(Guess, BadGuesses);

		} while (!Won && BadGuesses < 7); // loop until won or lost

		//
		// output end of game message
		//
		if (Won)
		{
			cout << endl << "You saved the poor man!  You won!" << endl;
		}
		else
		{
			cout << endl << "The word was \"" << Word
				<< "\"! The hangman has been hung!  You lose!" << endl;
			Losses++;
		}

		//
		// output play again menu
		//

		cout << "Play again (y/n)?" << endl;
		cout.flush();

		do
		{
			// gets a user-pressed key and makes it lowercase just in case
			PlayAgain = tolower( (char)getch() );

		} while (PlayAgain != 'y' && PlayAgain != 'n');

	} while (PlayAgain == 'y');

	//
	// free any allocated memory
	//
	if (Guess != NULL)
		free(Guess);

	if (Word != NULL)
		free(Word);

	//
	// close text file
	//
	InFile.close();

	cout << "Thanks for playing!  You hung " << Losses << " people!" << endl;

	return 0;

} // main()

//_[PickWord()]_________________________________________________________________
//
// Randomly picks a word from the specified input stream and returns it in a
// newly allocated character array (using 'malloc').  Returns a NULL on failure.
// This works by reading in the entire input file and then picking a random
// place inside it.  The nearest word is then chosen.
//
char *PickWord(ifstream &InFile)
{
	const int ShortestWord = 2;	// the minimum number of characters a valid ...
								// ... word may have
	const int JumpSize = 16;// amount of memory to allocate each time some ...
							// ... is needed
	int i;					// generic index variable
	int Position;			// position chosen in file
	int FileSize;			// size of file in bytes
	char *Buffer;			// buffer to store file contents
	char *Word = NULL;		// word to return
	int WordSpace;			// current memory available in 'Word'

	InFile.seekg(0, ios::end);

	if (!InFile.fail())
	{
		// get file size and jump back to beginning of file
		FileSize = InFile.tellg();
		InFile.seekg(0, ios::beg);

		// allocate buffer to store file
		Buffer = (char*)malloc(FileSize + 1);

		if (Buffer)
		{
			// read in entire file
			InFile.read(Buffer, FileSize);

			if (!InFile.fail())
			{
				// null-terminate buffer
				Buffer[FileSize] = '\0';

				// allocate initial space for word
				Word = (char*)malloc(JumpSize);
				WordSpace = JumpSize;

				//
				// loop until a word is found
				//
				do
				{
					// get random position
					Position = rand()%FileSize;

					// get previous word
					while (Position > 0 && isalpha(Buffer[Position]))
					{
						Position--;
					}

					// move past any white space
					while (!isalpha(Buffer[Position]) && Position < FileSize)
					{
						Position++;
					}

					// copy all the letters into Word, increasing the memory as
					// necessary
					for (i = 0; Word != NULL && Position < FileSize
								&& isalpha(Buffer[Position]); i++)
					{
						// if word needs more space for letters ... realloc
						if (i >= WordSpace)
						{
							WordSpace += JumpSize;
							Word = (char*)realloc(Word, WordSpace);

							if (Word != NULL)
							{
								Word[i] = Buffer[Position++];
							}
						}
						else
						{
							Word[i] = Buffer[Position++];
						}
					}

					// add terminating null character
					if (i >= WordSpace)
					{
						WordSpace += JumpSize;
						Word = (char*)realloc(Word, WordSpace);
					}

					if (Word != NULL)
					{
						Word[i] = '\0';
						i++;
					}

					// Continue looping if the word is too short (and there
					// is memory available, i.e. Word != NULL)
				} while (Word != NULL && strlen(Word) < ShortestWord);

			} // if read entire file

			// free buffer for file
			free(Buffer);

		} // if successfully allocated buffer

	} // if succeeded seeking to end

	// return found Word ... will contain NULL if any error occured INCLUDING
	// memory allocation errors.
	return Word;

} // PickWord()

//_[ClearScreen()]______________________________________________________________
//
// Clears the screen and places the cursor at the upper-left corner of the
// screen.
//
void ClearScreen(void)
{
	clrscr();

	gotoxy(0,0);

} // ClearScreen()

//_[DrawHangMan()]______________________________________________________________
//
// Draws a hangman based on the current number of bad guesses.
//
void DrawHangMan(int BadGuesses)
{
	const int MaxHeight = 4;// the maximum hangman height in lines
	int i;					// generic index variable
	int EndOfLine = 0;		// number of lines outputted

	//
	// draw each part of the hangman based on the number of bad guesses
	//
	for (i = 1; i <= BadGuesses; i++)
	{
		EndOfLine = 0;
		switch (i)
		{
			// head
			case 1:
				cout << " 0 " << endl;
				EndOfLine++;
				break;

			// left arm
			case 2:
				cout << "/";
				break;

			// neck
			case 3:
				cout << "|";
				break;

			// right arm
			case 4:
				cout << "\\" << endl;
				EndOfLine++;
				break;

			// body
			case 5:
				cout << " | " << endl;
				EndOfLine++;
				break;

			// left leg
			case 6:
				cout << "/ ";
				break;

			// right leg
			case 7:
				cout << "\\" << endl;
				EndOfLine++;
				break;
		}
	}

	// add extra empty lines to even out hangman
	while (EndOfLine < MaxHeight)
	{
		cout << endl;
		EndOfLine++;
	}

} // DrawHangMan()

//_[DrawGameScreen()]___________________________________________________________
//
// Draws the game screen.
//
void DrawGameScreen(char *Guess, int BadGuesses)
{
	int i;					// generic index variable

	// clear the screen to drop top to bottom
	ClearScreen();

	cout << "Guess the Word:  ";

	// display each letter guessed or 'X'
	for (i = 0; Guess[i] != '\0'; i++)
	{
		// unguessed letters are X's
		if (Guess[i] == ' ')
			cout << 'X';
		else
			cout << Guess[i];
	}

	// draw the hangman
	cout << endl << endl;
	DrawHangMan(BadGuesses);
	cout.flush();

} // DrawGameScreen