//*[main.cpp]*******************************************************************
//
// By Neil C. Obremski
//
// Purpose:
//  Write a program that will count how many random numbers need to be generated
//  in order to come up with 15 unique numbers in the range of 1-15. Store each
//  unique number in an Array as it is generated.  When all are generated, print
//  these 15 numbers in 2 columns with the index it is in being part of the
//  display and under the 2 columns with print how many random numbers were
//  generated in order to get the 15 unique ones.  For example:
//
//  [ 0]  3          [ 8]  2
//  [ 1] 14          [ 9] 11
//  [ 2]  8          [10] 16
//  [ 3] 15          [11] 9
//
//  385 random numbers generated in all!
//******************************************************************************

//_[header files]_______________________________________________________________
#include <iostream.h>		// for 'cout'/'cin'
#include <stdlib.h>			// for 'rand()'

//_[function prototypes]________________________________________________________
int RandomNumber(int iMin, int iMax);
void OutputResults(int iTotal, int aiArr[]);
int IsUnique(int iNumber, int aiArr[], int iMax);

//_[main()]_____________________________________________________________________
//
// Program entry point.
//
int main()
{
	//-[local variables]--------------------------------------------------------
	int iSeed,				// used to seed random number generator
		iTotalRand,			// total count of random numbers generated
		aiArray[15],		// array to hold numbers generated
		i,					// index variable used with 'aiArray'
		iRand;				// stores random number result from 'RandNum()'

	//-[seed random number generator]-------------------------------------------
	cout << "Input random number seed: ";
	cin >> iSeed;
	srand(iSeed);

	//-[generate a random number for each array subscript (15)]-----------------
	iTotalRand = 0;
	
	for (i = 0; i < 15; i++)
	{
		//-[loop until a unique number is found]--------------------------------
		do
		{
			iRand = RandomNumber(1, 15);

			iTotalRand++;

		} while (!IsUnique(iRand, aiArray, i));

		aiArray[i] = iRand;
		
	} // for

	//_[output results]_________________________________________________________
	OutputResults(iTotalRand, aiArray);

	return 0;
}

//_[RandomNumber()]_____________________________________________________________
//
// Generates a random number in the range passed in.  The random number may be
// equal to or greater than the minimum (iMin) and/or equal to or less than the
// maximum (iMax).  That is, if the function is used with 1 as a minimum and
// 15 as a maximum, the return will be anywhere from 1 to 15.
//
// The random number is generated using 'rand()' which is included from
// 'stdlib.h'.  The result of rand() can be any valid 'int' value.  I apply
// modulus (%) to the result to force it to be anywhere from zero up to one
// minus the maximum I specify.  The maximum range applied is 'iMax' minus
// 'iMin' plus one.  That is the total possible amount of unique random
// numbers within the range specified.  Lastly, the result of 'rand()' and
// 'modulus' is added to 'iMin' so that the number returned is within the
// caller-specified range.
//
// Parameters:
//  iMin .................. minimum number to be returned
//  iMax .................. maximum number to be returned
//
// Return (int):
//  Random number generated in the range of 'iMin' to 'iMax'.
//
int RandomNumber(int iMin, int iMax)
{
	return (rand()%(iMax-iMin+1)) + iMin;
}

//_[OutputResults()]____________________________________________________________
//
// Outputs the values of the array passed in as well as the total.  The array
// values are shown in two columns along with their respective indices.
//
// Parameters:
//  iTotal ................ the total number of random numbers generated
//  aiArr ................. array of unique random numbers generated
//
void OutputResults(int iTotal, int aiArr[])
{
	int i;

	//-[loop through half of the subscripts to print in two columns]------------
	for (i = 0; i < 8; i++)
	{
		cout << '[' << i << "] " << aiArr[i];

		//-[make sure second column exists for this index]----------------------
		if (i+8 < 15)
		{
			cout << "\t[" << (i+8) << "] " << aiArr[i+8];
		}

		cout << endl;
	}

	cout << endl << iTotal << " random numbers generated in all!" << endl;
}

//_[IsUnique()]_________________________________________________________________
//
// Determines if the specified number (iNumber) is present in the array (aiArr)
// in a certain number of subscripts (iMax).
//
// Parameters:
//  iNumber ............... number to check uniqueness of
//  aiArr ................. array to look for number in
//  iMax .................. number of subscript indices to check
//
// Return (int):
//  True (1) if the number does NOT exist in the array and is unique, false
//  otherwise.
//
int IsUnique(int iNumber, int aiArr[], int iMax)
{
	int i;

	for (i = 0; i < iMax; i++)
	{
		if (aiArr[i] == iNumber)
			return 0;
	}

	return 1;
}