//------------------------------------------------------------------------------
// File: shapes.cpp
// Desc: Shapes program; includes all shape classes and allows the user to
//		enter arbitrary sizes for calculations.
//
// Note: Source written with tabs set at 4 spaces.
//------------------------------------------------------------------------------

//_[headers]____________________________________________________________________
//
#include <iostream.h>		// i/o streaming (cout,cin)
#include "shape.h"			// shape class
#include "square.h"			// square class
#include "circle.h"			// circle class
#include "triangle.h"		// triangle class
#include "cube.h"			// cube class
#include "cylinder.h"		// cylinder class
#include "wedge.h"			// wedge class

//_[ShapeID enum]_______________________________________________________________
//
// Each shape class is entered in this enumeration and given a number by which
// to identify it.  This number is used by the user to select the shape.
//
enum ShapeID
{
	SquareID	= 0,
	CircleID	= 1,
	TriangleID	= 2,
	CubeID		= 3,
	CylinderID	= 4,
	WedgeID		= 5
};

//_[program entry point]________________________________________________________
//
//
int main()
{
	shape *pShape = 0;		// generic pointer to a shape object
	int Selection = 0;		// menu selection

	//
	// perform wanted calculations
	//
	// Kevin, I saw these in your initial email and I wasn't sure if you wanted
	// the program to perform those calculations explicitely.  If not, just
	// remove everything from this comment down to the 'output menu listing'
	// comment below.
	//
	square sq(10.0);
	circle ci(12.3);
	triangle tri(3.0,4.0,5.0);
	cube cu(10.0);
	cylinder cy(12.3,42.1);
	wedge w(3.0,4.0,5.0,14.3);

	sq.print();
	ci.print();
	tri.print();
	cu.print();
	cy.print();
	w.print();

	//
	// output menu listing
	//
	cout << "Please select a shape using it's ID: " << endl
		<< "[" << SquareID << "] Square" << endl
		<< "[" << CircleID << "] Circle" << endl
		<< "[" << TriangleID << "] Triangle" << endl
		<< "[" << CubeID << "] Cube" << endl
		<< "[" << CylinderID << "] Cylinder" << endl
		<< "[" << WedgeID << "] Wedge" << endl;

	//
	// output prompt and input user's selection
	//
	cout << "Shape> ";
	cin >> Selection;

	//
	// get additional input based on selection and create shape
	//
	switch (Selection)
	{
		//
		// square
		//
		case SquareID:
		{
			float Side;

			cout << "Enter single side of square> ";
			cin >> Side;

			pShape = new square(Side);

		}	break;

		//
		// circle
		//
		case CircleID:
		{
			float Radius;

			cout << "Enter radius of circle> ";
			cin >> Radius;

			pShape = new circle(Radius);

		}	break;

		//
		// triangle
		//
		case TriangleID:
		{
			float Side1, Side2, Side3;

			cout << "Enter side 1 of triangle> ";
			cin >> Side1;

			cout << "Enter side 2 of triangle> ";
			cin >> Side2;

			cout << "Enter side 3 of triangle> ";
			cin >> Side3;

			pShape = new triangle(Side1, Side2, Side3);

		}	break;

		//
		// cube
		//
		case CubeID:
		{
			float Side;

			cout << "Enter single side of cube> ";
			cin >> Side;

			pShape = new cube(Side);

		}	break;

		//
		// cylinder
		//
		case CylinderID:
		{
			float Radius, Height;

			cout << "Enter radius of cylinder> ";
			cin >> Radius;

			cout << "Enter height of cylinder> ";
			cin >> Height;

			pShape = new cylinder(Radius, Height);

		}	break;

		//
		// wedge
		//
		case WedgeID:
		{
			float Side1, Side2, Side3, Height;

			cout << "Enter side 1 of wedge> ";
			cin >> Side1;

			cout << "Enter side 2 of wedge> ";
			cin >> Side2;

			cout << "Enter side 3 of wedge> ";
			cin >> Side3;

			cout << "Height of wedge> ";
			cin >> Height;

			pShape = new wedge(Side1, Side2, Side3, Height);

		}	break;

		//
		// unknown!
		//
		default:
			cout << "Unknown shape id!" << endl;
	}

	//
	// if a shape was created (pShape is not NULL) ...
	//
	if (pShape)
	{
		// perform calculations and print that as well as shape information
		pShape->print();

		// remove shape object
		delete pShape;
	}

	return 0;
}