/*****************************************************************************

	[list.h] Template List class definition
	By Julio Arias

*****************************************************************************/
#ifndef LIST_H
#define LIST_H

#include <iostream>
#include <cassert>
#include "listnd.h"

using std::cout;

template< class NODETYPE >
class List {
public:
	List();							// constructor
	~List();						// destructor
	void insertAtFront( const NODETYPE & );
	void insertAtBack( const NODETYPE & );
	bool removeFromFront( NODETYPE & );
	bool removeFromBack( NODETYPE & );
	bool isEmpty() const;
	void print() const;
	long count() const;
	NODETYPE &top() const;

private:
	ListNode< NODETYPE > *firstPtr;	// pointer to first node
	ListNode< NODETYPE > *lastPtr;	// pointer to last node

	ListNode< NODETYPE > *getNewNode( const NODETYPE & );
};

//_[List::List()]_____________________________________________________________
//
// DESCRIPTION:
//  Default constructor
//
template< class NODETYPE >
List< NODETYPE >::List() : firstPtr( 0 ), lastPtr( 0 ) { }

//_[List::~List()]____________________________________________________________
//
// DESCRIPTION:
//  Destructor
//
template< class NODETYPE >
List< NODETYPE >::~List()
{
	if ( !isEmpty() ) {		// List is not empty
		
		ListNode< NODETYPE > *currentPtr = firstPtr, *tempPtr;
		
		while ( currentPtr != 0 ) {	// delete remaining nodes
			tempPtr = currentPtr;
			currentPtr = currentPtr->nextPtr;
			delete tempPtr;
		}
	}
}

//_[List::insertAtFront()]____________________________________________________
//
// DESCRIPTION:
//  Insert a node at the front of the list
//
template< class NODETYPE >
void List< NODETYPE >::insertAtFront( const NODETYPE &value )
{
	ListNode< NODETYPE > *newPtr = getNewNode( value );

	if ( isEmpty() )		// List is empty
		firstPtr = lastPtr = newPtr;
	else {					// List is not empty
		newPtr->nextPtr = firstPtr;
		firstPtr = newPtr;
	}
}

//_[List::insertAtBack()]_____________________________________________________
//
// DESCRIPTION:
//  Insert a node at the back of the list
//
template< class NODETYPE >
void List< NODETYPE >::insertAtBack( const NODETYPE &value )
{
	ListNode< NODETYPE > *newPtr = getNewNode( value );
	
	if ( isEmpty() )		// List is empty
		firstPtr = lastPtr = newPtr;
	else {					// List is not empty
		lastPtr->nextPtr = newPtr;
		lastPtr = newPtr;
	}
}

//_[List::removeFromFront()]__________________________________________________
//
// DESCRIPTION:
//  Delete a node from the front of the list
//
template< class NODETYPE >
bool List< NODETYPE >::removeFromFront( NODETYPE &value )
{
	if ( isEmpty() )			// List is empty
		return false;			// delete unsuccessful
	else {
		ListNode< NODETYPE > *tempPtr = firstPtr;
		
		if ( firstPtr == lastPtr )
			firstPtr = lastPtr = 0;
		else
			firstPtr = firstPtr->nextPtr;
		
		value = tempPtr->data;	// data being removed
		delete tempPtr;
		return true;			// delete successful
	}
}

//_[List::removeFromBack()]___________________________________________________
//
// DESCRIPTION:
//  Delete a node from the back of the list
//
template< class NODETYPE >
bool List< NODETYPE >::removeFromBack( NODETYPE &value )
{
	if ( isEmpty() )
		return false;			// delete unsuccessful
	else {
		ListNode< NODETYPE > *tempPtr = lastPtr;

		if ( firstPtr == lastPtr )
			firstPtr = lastPtr = 0;
		else {
			ListNode< NODETYPE > *currentPtr = firstPtr;
			
			while ( currentPtr->nextPtr != lastPtr )
				currentPtr = currentPtr->nextPtr;
			
			lastPtr = currentPtr;
			currentPtr->nextPtr = 0;
		}
		
		value = tempPtr->data;
		delete tempPtr;
		return true;			// delete successful
	}
}

//_[List::isEmpty()]__________________________________________________________
//
// DESCRIPTION:
//  Is the List empty?
//
template< class NODETYPE >
bool List< NODETYPE >::isEmpty() const
	{ return firstPtr == 0; }

//_[List::getNewNode()]_______________________________________________________
//
// DESCRIPTION:
//  Return a pointer to a newly allocated node
//
template< class NODETYPE >
ListNode< NODETYPE > *List< NODETYPE >::getNewNode( const NODETYPE &value )
{
	ListNode< NODETYPE > *ptr =
		new ListNode< NODETYPE >( value );
	assert( ptr != 0 );
	return ptr;
}

//_[List::print()]____________________________________________________________
//
// DESCRIPTION:
//  Display the contents of the List
//
template< class NODETYPE >
void List< NODETYPE >::print() const
{
	if ( isEmpty() ) {
		cout << "The list is empty\n\n";
		return;
	}

	ListNode< NODETYPE > *currentPtr = firstPtr;

	cout << "The list is: ";

	while ( currentPtr != 0 ) {
		cout << currentPtr->data << ' ';
		currentPtr = currentPtr->nextPtr;
	}

	cout << "\n\n";
}

//_[List::count()]____________________________________________________________
//
// DESCRIPTION:
//  Returns the number of items in the list.
//
template< class NODETYPE >
long List< NODETYPE >::count() const
{
	long iTotal = 0;

	ListNode< NODETYPE > *currentPtr = firstPtr;

	while ( currentPtr != 0 ) {
		iTotal++;
		currentPtr = currentPtr->nextPtr;
	}

	return iTotal;
}

//_[List::top()]______________________________________________________________
//
// DESCRIPTION:
//  Returns reference to the top-most item in the list.
//
template< class NODETYPE >
NODETYPE &List< NODETYPE >::top() const
{
	return firstPtr->data;
}

#endif