//*[xref.cpp]*******************************************************************
//
// Xref Class Definition
// By Neil C. Obremski
//
//******************************************************************************
#include "xref.h"			// Xref Class Prototype
#include <iostream>			// cout, cin

using namespace std;

//_[Xref::Xref()]_______________________________________________________________
//
// Default constructor.
//
Xref::Xref()
{
	std::cout << "Constructing Xref" << std::endl;
}

//_[Xref::~Xref()]______________________________________________________________
//
// Destructor.
//
Xref::~Xref()
{
	std::cout << "Destroying Xref" << std::endl;
}

//_[Xref::LoadWords()]__________________________________________________________
//
// Loads and parses the words from console input (cin).  If no input is pending,
// the function goes into "user-input" mode which allows the user to type in
// as many lines as they wish.
//
// Words are converted entirely to lower-case when parsed in order for the
// process to be insensitive to case.
//
// All non-alphabetic characters are seen as "word" separators and are not
// stored as words.  These non-alphabetic characters are detected using the
// isalpha() function which returns zero (0) if the specified character is not
// alphabetic.
//
// Pre-Conditions: none.
// Post-Conditions: none.
//
void Xref::LoadWords(void)
{
	m_Words.Clear();

	// If there are no characters available in the buffer.
	if (!cin.eof())
	{
		InputWordsFromUser_();
	}
	else
	{
		// The line buffer will be used to store the characters for each line
		// of the input.
		char sLineBuf[4096];
		int iLine = 0;

		for (iLine = 1; !std::cin.eof(); iLine++)
		{
			std::cin.getline(sLineBuf, 4096);

			m_Words.ParseLine(sLineBuf, iLine);
		}
	}
}

//_[Xref::OutputCrossReference()]_______________________________________________
//
// Outputs the entire cross reference table.
//
// Pre-Conditions: none.
// Post-Conditions: none.
//
void Xref::OutputCrossReference(void)
{
	char	sWord[128]	= {0};
	int		iFirst		= 0,
			iCount		= 0,
			iLine		= 0;

	m_Words.SortAscending();

	if (m_Words.GetFirstWord(sWord, 128))
	{
		do
		{
			m_Words.GetWord(sWord, 128);

			iCount = m_Words.GetLineCount();

			std::cout << sWord << '(' << iCount << ")\t\t";

			// Enumerate all the lines of the selected word.
			for (iFirst = m_Words.GetFirstLine();
				(iLine = m_Words.GetLine()) >= 1;
				m_Words.ToNextLine())
			{
				if (iFirst != iLine && iCount > 1)
					std::cout << ", ";
				std::cout << '(' << iLine << ")";
			}

			std::cout << endl;
			
		} while (m_Words.ToNextWord());
	} // if there are words (got first word)
	else
	{
		std::cout << "No words!" << std::endl;
	} // else if there are no words
}

//_[Xref::InputWordsFromUser()]_________________________________________________
//
// This is a private function that is called to "load" the words from user
// input rather than pending input (as from a file).
//
// Pre-Conditions: none.
// Post-Conditions: none.
//
void Xref::InputWordsFromUser_(void)
{
	std::cout
			<< "Enter text to be parsed and cross-referenced." << std::endl
			<< "Press [enter] on a blank line when you are finished." << std::endl;

	char sLineBuf[4096];
	int iLine = 0;

	do
	{
		iLine++;

		std::cout << "> ";

		std::cin.getline(sLineBuf, 4096);

		m_Words.ParseLine(sLineBuf, iLine);

	} while (*sLineBuf);
}

