//*[xreflist.h]*****************************************************************
//
// XrefList Class Prototype
// By Neil C. Obremski
//
//******************************************************************************
#ifndef __XREFLIST_CLASS
#define __XREFLIST_CLASS

//_[XrefList]___________________________________________________________________
//
// Class to store a two dimensional list of words and their associated line
// numbers.
//
class XrefList
{
public:
	XrefList();
	~XrefList();

	void Clear(void);

	bool ParseLine(char*,int);
	bool AddWord(char*,int);

	bool GetFirstWord(char*,int);
	bool GetWord(char*,int);
	bool ToNextWord(void);

	int GetLineCount(void);
	int GetFirstLine(void);
	int GetLine(void);
	bool ToNextLine(void);

	bool SortAscending(void);

private:

	//-[LineNode]---------------------------------------------------------------
	// Represents a single node in a singly linked-list of line numbers.
	//
	struct LineNode
	{
		LineNode *pNext;
		int iLine;
	}

	*m_pSelLine;	// the current line node selected (when enumerating lines)

	//-[WordNode]---------------------------------------------------------------
	// Structure that represents a single word as a node in a singly linked
	// list.  Each word node contains a pointer to the first line node in
	// another singly linked list to store line numbers (see 'LineNode'
	// structure).
	//
	struct WordNode
	{
		char *sText;
		WordNode *pNext;
		LineNode *pLines;
	}

	*m_pWords,		// first node in list of all words
	*m_pSelWord;	// the current word node selected (when enumerating words)

private:
	bool AddLineToWord_(WordNode*, int);

};

#endif